import json

from restic.internal import command_executor


class Error(Exception):
    pass


class UnexpectedResticResult(Error):
    pass


def run(restic_base_command,
        paths,
        exclude_patterns=None,
        exclude_files=None,
        dry_run=None):
    cmd = restic_base_command + ['backup'] + paths

    if exclude_patterns:
        for exclude_pattern in exclude_patterns:
            cmd.extend(['--exclude', exclude_pattern])

    if exclude_files:
        for exclude_file in exclude_files:
            cmd.extend(['--exclude-file', exclude_file])

    if dry_run:
        cmd.extend(['--dry-run'])

    result_raw = command_executor.execute(cmd)
    return _parse_result(result_raw)


def _parse_result(result):
    # On Windows, terminal markers appear at the beginning of each line.
    terminal_markers = '\x1b[2K'
    lines = [
        line.strip().strip(terminal_markers)
        for line in result.split('\n')
        if line.strip()
    ]

    try:
        return json.loads(lines[-1])
    except json.decoder.JSONDecodeError as e:
        raise UnexpectedResticResult(
            'Expected valid JSON response from restic, got %s' % result) from e



import json

import restic.errors
from restic.internal import command_executor


def run(restic_base_command,
        prune=False,
        keep_daily=None,
        keep_within=None,
        group_by=None):
    cmd = restic_base_command + ['forget']

    if prune:
        cmd.extend(['--prune'])

    if keep_daily is not None:
        cmd.extend(['--keep-daily', str(keep_daily)])

    if keep_within is not None:
        cmd.extend(['--keep-within', keep_within])

    if group_by is not None:
        cmd.extend(['--group-by', group_by])

    return _parse_result(command_executor.execute(cmd))


def _parse_result(result):
    # The result is JSON followed by 0 or more non-JSON lines.
    result_lines = result.split('\n')
    try:
        return json.loads(result_lines[0])
    except json.decoder.JSONDecodeError as e:
        raise restic.errors.UnexpectedResticResponse(
            'Unexpected result from restic. Expected JSON, got: %s' %
            result_lines[0]) from e



from restic.internal import backup as internal_backup
from restic.internal import check as internal_check
from restic.internal import copy as internal_copy
from restic.internal import forget as internal_forget
from restic.internal import generate as internal_generate
from restic.internal import init as internal_init
from restic.internal import restore as internal_restore
from restic.internal import self_update as internal_self_update
from restic.internal import snapshots as internal_snapshots
from restic.internal import stats as internal_stats
from restic.internal import unlock as internal_unlock
from restic.internal import version as internal_version

# Ignore warnings about naming of globals.
# pylint: disable=C0103
binary_path = 'restic'

# Global flags
# Ignore warnings about naming of globals.
# pylint: disable=C0103
repository = None
password_file = None


def backup(*args, **kwargs):
    return internal_backup.run(_make_base_command(), *args, **kwargs)


def check(*args, **kwargs):
    return internal_check.run(_make_base_command(), *args, **kwargs)


def copy(*args, **kwargs):
    return internal_copy.run(_make_base_command(), *args, **kwargs)


def forget(*args, **kwargs):
    return internal_forget.run(_make_base_command(), *args, **kwargs)


def generate(*args, **kwargs):
    return internal_generate.run(_make_base_command(), *args, **kwargs)


def init(*args, **kwargs):
    return internal_init.run(_make_base_command(), *args, **kwargs)


def restore(*args, **kwargs):
    return internal_restore.run(_make_base_command(), *args, **kwargs)


def self_update():
    return internal_self_update.run(_make_base_command())


def snapshots(*args, **kwargs):
    return internal_snapshots.run(_make_base_command(), *args, **kwargs)


def stats(*args, **kwargs):
    return internal_stats.run(_make_base_command(), *args, **kwargs)


def unlock():
    return internal_unlock.run(_make_base_command())


def version():
    return internal_version.run(_make_base_command())


def _make_base_command():
    base_command = [binary_path]

    # Always add the JSON flag so we get back results in JSON.
    base_command.extend(['--json'])

    if repository:
        base_command.extend(['--repo', repository])

    if password_file:
        base_command.extend(['--password-file', password_file])

    return base_command



import re

from restic.internal import command_executor


def run(restic_base_command):
    cmd = restic_base_command + ['version']
    out = command_executor.execute(cmd)
    match = re.match((r'restic ([0-9\.]+) compiled with go([0-9\.]+) '
                      'on ([a-zA-Z0-9]+)/([a-zA-Z0-9]+)'), out)
    return {
        'restic_version': match.group(1),
        'go_version': match.group(2),
        'platform_version': match.group(3),
        'architecture': match.group(4)
    }



import unittest
from unittest import mock

import restic
from restic.internal import self_update

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class SelfUpdateTest(unittest.TestCase):

    @mock.patch.object(self_update.command_executor, 'execute')
    def test_self_update(self, mock_execute):
        restic.self_update()
        mock_execute.assert_called_with(['restic', '--json', 'self-update'])



from restic.internal import command_executor


def run(restic_base_command,
        bash_completion_path=None,
        man_directory=None,
        zsh_completion_path=None):
    cmd = restic_base_command + ['generate']

    if bash_completion_path is not None:
        cmd.extend(['--bash-completion', bash_completion_path])

    if man_directory is not None:
        cmd.extend(['--man', man_directory])

    if zsh_completion_path is not None:
        cmd.extend(['--zsh-completion', zsh_completion_path])

    return command_executor.execute(cmd)



import re

from restic.internal import command_executor


def run(restic_base_command, copy_chunker_params=False, repo2=None):
    cmd = restic_base_command + ['init']

    if copy_chunker_params:
        cmd.append('--copy-chunker-params')

    if repo2:
        cmd.extend(['--repo2', repo2])

    return _parse_result(command_executor.execute(cmd))


def _parse_result(result):
    return re.match(r'created restic repository ([a-z0-9]+) at .+',
                    result).group(1)



import unittest
from unittest import mock

import restic
from restic.internal import version


class VersionTest(unittest.TestCase):

    @mock.patch.object(version.command_executor, 'execute')
    def test_version(self, mock_execute):
        mock_execute.return_value = (
            'restic 0.12.0 compiled with go1.15.8 on windows/amd64')

        self.assertEqual(
            {
                'architecture': 'amd64',
                'go_version': '1.15.8',
                'platform_version': 'windows',
                'restic_version': '0.12.0'
            }, restic.version())

        mock_execute.assert_called_with(['restic', '--json', 'version'])



import unittest
from unittest import mock

import restic
from restic.internal import snapshots

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class SnapshotsTest(unittest.TestCase):

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_simple(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.snapshots()

        mock_execute.assert_called_with(['restic', '--json', 'snapshots'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_group_by_host(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.snapshots(group_by='host')

        mock_execute.assert_called_with(
            ['restic', '--json', 'snapshots', '--group-by', 'host'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_parses_result_json(self, mock_execute):
        mock_execute.return_value = """
[
  {
    "group_key": {
      "hostname": "ace809d23440",
      "paths": null,
      "tags": null
    },
    "snapshots": [
      {
        "gid": 3434,
        "hostname": "ace809d23440",
        "id": "bbe7f04941ed969ee940bb41dc04196027fcfb83dbdfea93c16afb2cb9f6dd81",
        "paths": [
          "/tmp/tmp6594jneh/mydata.txt"
        ],
        "short_id": "bbe7f049",
        "time": "2021-03-28T21:31:44.477122573Z",
        "tree": "f589421bafdae95f5be5eea6285074b7ddc54aa0ffd1ad606f74d1e6207d20a3",
        "uid": 3434,
        "username": "circleci"
      }
    ]
  }
]
""".strip()

        # Ignore complaint about too long of a line.
        # pylint: disable=C0301
        self.assertEqual([{
            'group_key': {
                'hostname': 'ace809d23440',
                'paths': None,
                'tags': None
            },
            'snapshots': [{
                'time':
                    '2021-03-28T21:31:44.477122573Z',
                'tree':
                    'f589421bafdae95f5be5eea6285074b7ddc54aa0ffd1ad606f74d1e6207d20a3',
                'paths': ['/tmp/tmp6594jneh/mydata.txt'],
                'hostname':
                    'ace809d23440',
                'username':
                    'circleci',
                'uid':
                    3434,
                'gid':
                    3434,
                'id':
                    'bbe7f04941ed969ee940bb41dc04196027fcfb83dbdfea93c16afb2cb9f6dd81',
                'short_id':
                    'bbe7f049'
            }]
        }], restic.snapshots())



import unittest
from unittest import mock

import restic
from restic.internal import generate

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class GenerateTest(unittest.TestCase):

    @mock.patch.object(generate.command_executor, 'execute')
    def test_generate_with_no_parameters(self, mock_execute):
        restic.generate()
        mock_execute.assert_called_with(['restic', '--json', 'generate'])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_generate_bash(self, mock_execute):
        restic.generate(bash_completion_path='/tmp/dummy.path')
        mock_execute.assert_called_with([
            'restic', '--json', 'generate', '--bash-completion',
            '/tmp/dummy.path'
        ])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_generate_man(self, mock_execute):
        restic.generate(man_directory='/tmp/man-dir')
        mock_execute.assert_called_with(
            ['restic', '--json', 'generate', '--man', '/tmp/man-dir'])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_generate_zsh(self, mock_execute):
        restic.generate(zsh_completion_path='/tmp/dummy.path')
        mock_execute.assert_called_with([
            'restic', '--json', 'generate', '--zsh-completion',
            '/tmp/dummy.path'
        ])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_generate_all(self, mock_execute):
        restic.generate(bash_completion_path='/tmp/dummy.path',
                        man_directory='/tmp/man-dir',
                        zsh_completion_path='/tmp/dummy.path')
        mock_execute.assert_called_with([
            'restic', '--json', 'generate', '--bash-completion',
            '/tmp/dummy.path', '--man', '/tmp/man-dir', '--zsh-completion',
            '/tmp/dummy.path'
        ])



import unittest
from unittest import mock

import restic
from restic.internal import backup

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class BackupTest(unittest.TestCase):

    @mock.patch.object(backup.command_executor, 'execute')
    def test_backup_single_path(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/tmp/dummy-file.txt'])

        mock_execute.assert_called_with(
            ['restic', '--json', 'backup', '/tmp/dummy-file.txt'])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_backup_multiple_paths(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/tmp/dummy-file-1.txt', '/tmp/dummy-file-2.txt'])

        mock_execute.assert_called_with([
            'restic', '--json', 'backup', '/tmp/dummy-file-1.txt',
            '/tmp/dummy-file-2.txt'
        ])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_excludes_single_pattern(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/data/music'], exclude_patterns=['Justin Bieber*'])

        mock_execute.assert_called_with([
            'restic', '--json', 'backup', '/data/music', '--exclude',
            'Justin Bieber*'
        ])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_excludes_multiple_patterns(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/data/music'],
                      exclude_patterns=['Justin Bieber*', 'Selena Gomez*'])

        mock_execute.assert_called_with([
            'restic', '--json', 'backup', '/data/music', '--exclude',
            'Justin Bieber*', '--exclude', 'Selena Gomez*'
        ])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_dry_run(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/data/music'], dry_run=True)

        mock_execute.assert_called_with(
            ['restic', '--json', 'backup', '/data/music', '--dry-run'])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_excludes_single_exclude_file(self, mock_execute):
        mock_execute.return_value = '{}'

        restic.backup(['/data/music'], exclude_files=['bad-songs.txt'])

        mock_execute.assert_called_with([
            'restic', '--json', 'backup', '/data/music', '--exclude-file',
            'bad-songs.txt'
        ])

    @mock.patch.object(backup.command_executor, 'execute')
    def test_parses_result_json(self, mock_execute):
        mock_execute.return_value = """
{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20}
{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20}
{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
{"message_type":"status","percent_done":1,"total_files":1,"total_bytes":20,"bytes_done":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
{"message_type":"summary","files_new":1,"files_changed":0,"files_unmodified":0,"dirs_new":2,"dirs_changed":0,"dirs_unmodified":0,"data_blobs":1,"tree_blobs":3,"data_added":1115,"total_files_processed":1,"total_bytes_processed":20,"total_duration":0.216764185,"snapshot_id":"01d88ea7"}
""".lstrip()
        backup_summary = restic.backup(['/tmp/dummy-file.txt'])
        self.assertEqual(
            {
                'message_type': 'summary',
                'files_new': 1,
                'files_changed': 0,
                'files_unmodified': 0,
                'dirs_new': 2,
                'dirs_changed': 0,
                'dirs_unmodified': 0,
                'data_blobs': 1,
                'tree_blobs': 3,
                'data_added': 1115,
                'total_files_processed': 1,
                'total_bytes_processed': 20,
                'total_duration': 0.216764185,
                'snapshot_id': '01d88ea7'
            }, backup_summary)

    @mock.patch.object(backup.command_executor, 'execute')
    def test_parses_result_json_ignores_terminal_markers(self, mock_execute):
        mock_execute.return_value = """
\x1b[2K{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20}
\x1b[2K{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20}
\x1b[2K{"message_type":"status","percent_done":0,"total_files":1,"total_bytes":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"total_bytes":20,"bytes_done":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20,"current_files":["/tmp/tmpvg2jkmqw/mydata.txt"]}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
\x1b[2K{"message_type":"status","percent_done":1,"total_files":1,"files_done":1,"total_bytes":20,"bytes_done":20}
\x1b[2K{"message_type":"summary","files_new":1,"files_changed":0,"files_unmodified":0,"dirs_new":2,"dirs_changed":0,"dirs_unmodified":0,"data_blobs":1,"tree_blobs":3,"data_added":1115,"total_files_processed":1,"total_bytes_processed":20,"total_duration":0.216764185,"snapshot_id":"01d88ea7"}
""".lstrip()
        backup_summary = restic.backup(['/tmp/dummy-file.txt'])
        self.assertEqual(
            {
                'message_type': 'summary',
                'files_new': 1,
                'files_changed': 0,
                'files_unmodified': 0,
                'dirs_new': 2,
                'dirs_changed': 0,
                'dirs_unmodified': 0,
                'data_blobs': 1,
                'tree_blobs': 3,
                'data_added': 1115,
                'total_files_processed': 1,
                'total_bytes_processed': 20,
                'total_duration': 0.216764185,
                'snapshot_id': '01d88ea7'
            }, backup_summary)

    @mock.patch.object(backup.command_executor, 'execute')
    def test_wraps_non_json_response(self, mock_execute):
        mock_execute.return_value = '[[invalid response]]'

        with self.assertRaises(backup.UnexpectedResticResult):
            restic.backup(['/tmp/dummy-file.txt'])



import unittest
from unittest import mock

import restic.errors
from restic.internal import forget

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class ForgetTest(unittest.TestCase):

    def setUp(self):
        self.maxDiff = None

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget()
        mock_execute.assert_called_with(['restic', '--json', 'forget'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_and_prune(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget(prune=True)
        mock_execute.assert_called_with(
            ['restic', '--json', 'forget', '--prune'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_all_except_last_30_days(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget(keep_daily=30)
        mock_execute.assert_called_with(
            ['restic', '--json', 'forget', '--keep-daily', '30'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_all_except_last_30_days_and_prune(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget(prune=True, keep_daily=30)
        mock_execute.assert_called_with(
            ['restic', '--json', 'forget', '--prune', '--keep-daily', '30'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_with_group_by(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget(prune=True, group_by='host')
        mock_execute.assert_called_with(
            ['restic', '--json', 'forget', '--prune', '--group-by', 'host'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_and_keep_within(self, mock_execute):
        mock_execute.return_value = '{}'
        restic.forget(prune=True, keep_within='60d')
        mock_execute.assert_called_with(
            ['restic', '--json', 'forget', '--prune', '--keep-within', '60d'])

    @mock.patch.object(forget.command_executor, 'execute')
    def test_parses_result_json(self, mock_execute):
        # Ignore complaint about too long of a line.
        # pylint: disable=C0301
        mock_execute.return_value = """
[{"tags": null, "host": "ecb5551395ae", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "keep": [{"time": "2021-03-16T00:10:37.015657013Z", "tree": "4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "hostname": "ecb5551395ae", "username": "circleci", "uid": 3434, "gid": 3434, "id": "3f6de49c6461ffd42900a204655708a3e136a3814abe298c07f27e412e2b6a43", "short_id": "3f6de49c"}], "remove": null, "reasons": [{"snapshot": {"time": "2021-03-16T00:10:37.015657013Z", "tree": "4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "hostname": "ecb5551395ae", "username": "circleci", "uid": 3434, "gid": 3434}, "matches": ["daily snapshot"], "counters": {"daily": 4}}]}]
""".strip()
        forget_result = restic.forget()
        self.assertEqual([{
            'tags':
                None,
            'host':
                'ecb5551395ae',
            'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
            'keep': [{
                'time':
                    '2021-03-16T00:10:37.015657013Z',
                'tree':
                    '4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f',
                'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
                'hostname':
                    'ecb5551395ae',
                'username':
                    'circleci',
                'uid':
                    3434,
                'gid':
                    3434,
                'id':
                    '3f6de49c6461ffd42900a204655708a3e136a3814abe298c07f27e412e2b6a43',
                'short_id':
                    '3f6de49c'
            }],
            'remove':
                None,
            'reasons': [{
                'snapshot': {
                    'time':
                        '2021-03-16T00:10:37.015657013Z',
                    'tree':
                        '4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f',
                    'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
                    'hostname':
                        'ecb5551395ae',
                    'username':
                        'circleci',
                    'uid':
                        3434,
                    'gid':
                        3434
                },
                'matches': ['daily snapshot'],
                'counters': {
                    'daily': 4
                }
            }]
        }], forget_result)

    @mock.patch.object(forget.command_executor, 'execute')
    def test_parses_result_json_and_ignores_text_afterwards(self, mock_execute):
        # Ignore complaint about too long of a line.
        # pylint: disable=C0301
        mock_execute.return_value = """
[{"tags": null, "host": "ecb5551395ae", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "keep": [{"time": "2021-03-16T00:10:37.015657013Z", "tree": "4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "hostname": "ecb5551395ae", "username": "circleci", "uid": 3434, "gid": 3434, "id": "3f6de49c6461ffd42900a204655708a3e136a3814abe298c07f27e412e2b6a43", "short_id": "3f6de49c"}], "remove": null, "reasons": [{"snapshot": {"time": "2021-03-16T00:10:37.015657013Z", "tree": "4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f", "paths": ["/tmp/tmp6ew1vzp2/mydata.txt"], "hostname": "ecb5551395ae", "username": "circleci", "uid": 3434, "gid": 3434}, "matches": ["daily snapshot"], "counters": {"daily": 4}}]}]
loading indexes...
loading all snapshots...
finding data that is still in use for 2 snapshots
[0:00] 100.00%  2 / 2 snapshots

searching used packs...
collecting packs for deletion and repacking
[0:00] 100.00%  6 / 6 packs processed


to repack:            0 blobs / 0 B
this removes          0 blobs / 0 B
to delete:            0 blobs / 0 B
total prune:          0 blobs / 0 B
remaining:           49 blobs / 4.890 MiB
unused size after prune: 0 B (0.00% of remaining size)

done
""".lstrip()
        forget_result = restic.forget()
        self.assertEqual([{
            'tags':
                None,
            'host':
                'ecb5551395ae',
            'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
            'keep': [{
                'time':
                    '2021-03-16T00:10:37.015657013Z',
                'tree':
                    '4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f',
                'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
                'hostname':
                    'ecb5551395ae',
                'username':
                    'circleci',
                'uid':
                    3434,
                'gid':
                    3434,
                'id':
                    '3f6de49c6461ffd42900a204655708a3e136a3814abe298c07f27e412e2b6a43',
                'short_id':
                    '3f6de49c'
            }],
            'remove':
                None,
            'reasons': [{
                'snapshot': {
                    'time':
                        '2021-03-16T00:10:37.015657013Z',
                    'tree':
                        '4483c2c6c1386abb9f47497cf108bab19e09c42430d32cd640a4f6f97137841f',
                    'paths': ['/tmp/tmp6ew1vzp2/mydata.txt'],
                    'hostname':
                        'ecb5551395ae',
                    'username':
                        'circleci',
                    'uid':
                        3434,
                    'gid':
                        3434
                },
                'matches': ['daily snapshot'],
                'counters': {
                    'daily': 4
                }
            }]
        }], forget_result)

    @mock.patch.object(forget.command_executor, 'execute')
    def test_forget_raises_exception_when_response_is_invalid_json(
            self, mock_execute):
        mock_execute.return_value = '{{{{{{{[[[[[['
        with self.assertRaises(restic.errors.UnexpectedResticResponse):
            restic.forget()



import unittest
from unittest import mock

import restic
from restic.internal import copy

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class CopyTest(unittest.TestCase):

    @mock.patch.object(copy.command_executor, 'execute')
    def test_copy_simple(self, mock_execute):
        mock_execute.return_value = """
snapshot 670792c6 of [/tmp/tmp9k613t9a/mydata.txt] at 2021-03-29 00:56:31.183738563 +0000 UTC)
  copy started, this may take a while...
snapshot 3671204c saved
""".strip()

        restic.copy(repo2='s3:https://dummyrepo.example.com/bucket',
                    password_file2='/dummy/pass.txt')

        mock_execute.assert_called_with([
            'restic', '--json', 'copy', '--repo2',
            's3:https://dummyrepo.example.com/bucket', '--password-file2',
            '/dummy/pass.txt'
        ])



import unittest
from unittest import mock

import restic
from restic.internal import init

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class InitTest(unittest.TestCase):

    @mock.patch.object(init.command_executor, 'execute')
    def test_unlock_with_no_parameters(self, mock_execute):
        restic.unlock()
        mock_execute.assert_called_with(['restic', '--json', 'unlock'])



import unittest
from unittest import mock

import restic
from restic.internal import stats

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class SelfUpdateTest(unittest.TestCase):

    @mock.patch.object(stats.command_executor, 'execute')
    def test_stats(self, mock_execute):
        mock_execute.return_value = """{
            "total_size": 20,
            "total_file_count": 3
            }
            """

        self.assertEqual({
            'total_size': 20,
            'total_file_count': 3
        }, restic.stats())

        mock_execute.assert_called_with(['restic', '--json', 'stats'])

    @mock.patch.object(stats.command_executor, 'execute')
    def test_stats_with_mode(self, mock_execute):
        mock_execute.return_value = """{
            "total_size": 20,
            "total_file_count": 1,
            "total_blob_count": 1
            }
            """

        self.assertEqual(
            {
                'total_size': 20,
                'total_file_count': 1,
                'total_blob_count': 1
            }, restic.stats(mode='blobs-per-file'))

        mock_execute.assert_called_with(
            ['restic', '--json', 'stats', '--mode', 'blobs-per-file'])



from restic.internal import command_executor


def run(restic_base_command, repo2=None, password_file2=None):
    cmd = restic_base_command + ['copy']

    if repo2:
        cmd.extend(['--repo2', repo2])

    if password_file2:
        cmd.extend(['--password-file2', password_file2])

    return command_executor.execute(cmd)



from restic.internal import command_executor


def run(restic_base_command, snapshot_id='latest', target_dir=None):
    cmd = restic_base_command + ['restore', snapshot_id]

    if target_dir:
        cmd.extend(['--target', target_dir])

    return command_executor.execute(cmd)



import unittest
from unittest import mock

import restic
from restic.internal import init

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class InitTest(unittest.TestCase):

    @mock.patch.object(init.command_executor, 'execute')
    def test_init_with_no_parameters(self, mock_execute):
        mock_execute.return_value = (
            'created restic repository 054ed643d8 at /media/backup1')

        repository_id = restic.init()

        self.assertEqual('054ed643d8', repository_id)
        mock_execute.assert_called_with(['restic', '--json', 'init'])

    @mock.patch.object(init.command_executor, 'execute')
    def test_init_with_secondary_repository(self, mock_execute):
        mock_execute.return_value = (
            'created restic repository 054ed643d8 at /media/backup1')

        repository_id = restic.init(
            copy_chunker_params=True,
            repo2='s3:https://some.backend.com/mybucket')

        self.assertEqual('054ed643d8', repository_id)
        mock_execute.assert_called_with([
            'restic', '--json', 'init', '--copy-chunker-params', '--repo2',
            's3:https://some.backend.com/mybucket'
        ])



import json

from restic.internal import command_executor


def run(restic_base_command, mode=None):
    cmd = restic_base_command + ['stats']

    if mode:
        cmd.extend(['--mode', mode])

    return json.loads(command_executor.execute(cmd))



import unittest
from unittest import mock

import restic
from restic.internal import generate

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class ResticTest(unittest.TestCase):

    def setUp(self):
        self.original_binary = restic.binary_path
        self.original_repository = restic.repository
        self.original_password_file = restic.password_file

    def tearDown(self):
        restic.binary_path = self.original_binary
        restic.repository = self.original_repository
        restic.password_file = self.original_password_file

    @mock.patch.object(generate.command_executor, 'execute')
    def test_can_change_restic_binary_path(self, mock_execute):
        restic.binary_path = '/dummy/path/to/restic-binary'
        restic.generate()
        mock_execute.assert_called_with(
            ['/dummy/path/to/restic-binary', '--json', 'generate'])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_can_set_repository_path(self, mock_execute):
        mock_execute.return_value = (
            'created restic repository 054ed643d8 at /media/backup1')

        restic.repository = '/dummy/repo/path'
        restic.init()

        mock_execute.assert_called_with(
            ['restic', '--json', '--repo', '/dummy/repo/path', 'init'])

    @mock.patch.object(generate.command_executor, 'execute')
    def test_can_set_password_file(self, mock_execute):
        mock_execute.return_value = (
            'created restic repository 054ed643d8 at /media/backup1')

        restic.password_file = 'secret-pw.txt'
        restic.init()

        mock_execute.assert_called_with(
            ['restic', '--json', '--password-file', 'secret-pw.txt', 'init'])



import unittest
from unittest import mock

import restic.errors
from restic.internal import check

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class CheckTest(unittest.TestCase):

    @mock.patch.object(check.command_executor, 'execute')
    def test_check_simple(self, mock_execute):
        restic.check()

        mock_execute.assert_called_with(['restic', '--json', 'check'])

    @mock.patch.object(check.command_executor, 'execute')
    def test_check_and_read_data(self, mock_execute):
        restic.check(read_data=True)

        mock_execute.assert_called_with(
            ['restic', '--json', 'check', '--read-data'])

    @mock.patch.object(check.command_executor, 'execute')
    def test_check_returns_none_on_restic_failure(self, mock_execute):
        mock_execute.side_effect = restic.errors.ResticFailedError(
            'dummy restic failure')

        self.assertIsNone(restic.check())



import restic.errors
from restic.internal import command_executor


def run(restic_base_command, read_data=False):
    cmd = restic_base_command + ['check']

    if read_data:
        cmd.append('--read-data')

    try:
        return command_executor.execute(cmd)
    except restic.errors.ResticFailedError:
        return None



import json

from restic.internal import command_executor


def run(restic_base_command, group_by=None):
    cmd = restic_base_command + ['snapshots']

    if group_by:
        cmd.extend(['--group-by', group_by])

    return json.loads(command_executor.execute(cmd))



#!/usr/bin/env python

import os.path

import setuptools

setuptools.setup(
    name='resticpy',
    long_description=open(
        os.path.join(os.path.abspath(os.path.dirname(__file__)),
                     'README.md')).read(),
    long_description_content_type="text/markdown",
    version='0.1.4',
    description='Restic backup Python wrapper',
    project_urls={
        "repository": "https://github.com/mtlynch/resticpy",
    },
    author='Michael Lynch',
    author_email='michael@mtlynch.io',
    license="MIT",
    keywords="backup",
    url='https://github.com/mtlynch/resticpy.git',
    packages=['restic', 'restic.internal'],
    install_requires=[],
    python_requires='>=3.7',
)



import logging
import subprocess

import restic.errors

logger = logging.getLogger(__name__)


def execute(cmd):
    logger.debug('Executing restic command: %s', str(cmd))
    try:
        process = subprocess.run(cmd,
                                 capture_output=True,
                                 text=True,
                                 encoding='utf-8')
    except FileNotFoundError as e:
        raise restic.errors.NoResticBinaryEror(
            'Cannot find restic installed') from e

    logger.debug('Restic command completed with return code %d',
                 process.returncode)

    if process.returncode != 0:
        raise restic.errors.ResticFailedError(
            'Restic failed with exit code %s: %s' %
            (process.returncode, process.stderr))

    return process.stdout



import unittest
from unittest import mock

import restic
from restic.internal import restore

# Ignore suggestions to turn methods into functions.
# pylint: disable=R0201


class RestoreTest(unittest.TestCase):

    @mock.patch.object(restore.command_executor, 'execute')
    def test_restore_with_no_snapshot_id(self, mock_execute):
        restic.restore()
        mock_execute.assert_called_with(
            ['restic', '--json', 'restore', 'latest'])

    @mock.patch.object(restore.command_executor, 'execute')
    def test_restore_specific_snapshot_id(self, mock_execute):
        restic.restore('dummy-snapshot-id')
        mock_execute.assert_called_with(
            ['restic', '--json', 'restore', 'dummy-snapshot-id'])

    @mock.patch.object(restore.command_executor, 'execute')
    def test_restore_specific_snapshot_id_and_target(self, mock_execute):
        restic.restore(snapshot_id='dummy-snapshot-id',
                       target_dir='/tmp/restore')
        mock_execute.assert_called_with([
            'restic', '--json', 'restore', 'dummy-snapshot-id', '--target',
            '/tmp/restore'
        ])
